#' Two and three-phase regression for discrete outcome
#'
#' Function that applies the Horvitz-Thompson, the conditional maximum likelihood and the joint modelling approaches, for both two and three-phase data. This is a first version of the function and accepts only a discrete outcome. Extensions are under implementation.
#' @param regformula Regression formula: cc ~ x1 + x2
#' @param desnformula1 Regression model for the first selection probability.
#' @param desnformula2 Regression model for the second selection probability.
#' @param data Dataset in matrix format, containing all fully observed (outcome 'cc' and 'cheap' variables) and partially observed variables. See example on how to write the dataset.
#' @param method Either 'offset', if the conditional maximum likelihood approach is to be used, or 'weighted', if the Horvitz-Thompson estimator is of interest.
#' @param tol Maximum tolerance for Newton-Raphson iteration.
#' @param maxstepprop Maximum step to be taken in the Newton-Raphson iteration.
#' @param addStilde TRUE, if the joint estimation method is selected, FALSE otherwise.
#' @param niter Default is 1. If addStilde = TRUE, niter must be changed to some value larger than 1.
#' @param print.iters Print steps, TRUE or FALSE
#' @keywords conditional maximum likelihood, Horvitz-Thompson, Two-phase sampling
#' @export
#' @examples
#' N <- 15000; beta <- c(-3,1,.5)
#' x1 <- rnorm(N)
#' x2 <- rnorm(N)
#' x1d <- 1*(x1<0)
#' x2d <- 1*(x2<.5)
#' ptrue <- plogis(beta[1] + beta[2]*x1 + beta[3]*x2)
#' cc <- 1*(runif(N)<ptrue)
#'
#' ##Totals
#' Ny1d0 <- sum(cc==1 & x1d==0); Ny1d1 <- sum(cc==1 & x1d==1);
#' Ny0d1 <- sum(cc==0 & x1d==1); Ny0d0 <- sum(cc==0 & x1d==0)
#'
#' ## Sample sizes
#' nsamples <- c(ny1d0=400, ny1d1=100, ny0d0=400, ny0d1=400)
#'
#' ## Selection
#' ind <- 1:N
#' samp1 <- c(sample(ind[cc==1 & x1d==0], nsamples[1]), sample(ind[cc==1 & x1d==1], nsamples[2]),
#'            sample(ind[cc==0 & x1d==0], nsamples[3]), sample(ind[cc==0 & x1d==1], nsamples[4]))
#'
#' Ntotals <- cbind(Ny1d0, Ny1d1, Ny0d0, Ny0d1)
#' names(nsamples) <- c("y1d0", "y1d1", "y0d0", "y0d1")
#'
#' R1 <- rep(0,N); R1[samp1] <- 1
#'
#' ## Response
#' samp2_func <- function(n,yu,v1,v2){sample(ind[R1==1 & cc==yu & x1d==v1 & x2d==v2],n)}
#' samp2 <- c(samp2_func(50,0,0,0),samp2_func(50,0,0,1),samp2_func(50,0,1,0),samp2_func(50,0,1,1),
#'            samp2_func(50,1,0,0),samp2_func(50,1,0,1),samp2_func(25,1,1,0),samp2_func(25,1,1,1))
#' R2 <- rep(NA,N);R2[R1==1] <- 0
#' R2[samp2] <- 1
#' x2d[R1==0] <- NA
#'
#' datcsv <- data.frame(R1=R1,R2=R2,cc=cc,x1=x1,x1d=x1d,x2=x2,x2d=x2d)
#'
#' regformula <- cc~x1+x2
#' desnformula1 <- R1~cc*x1d
#' desnformula2 <- R2~cc*x1d*x2d + cc*x1
#'
#' out <- func(regformula, desnformula1, desnformula2, desnformula3=NULL, data=datcsv, method="offset", addStilde=TRUE, niter=25)

func <- function(regformula, desnformula1, desnformula2=NULL, desnformula3=NULL, data, method="offset", addStilde=FALSE, niter=25, tol=1.0e-6, maxstepprop=0.1, print.iters=FALSE){
    error <- 0; ow <- options("warn"); mzero <- function(n,m) matrix(0,n,m) # utility function
    data$counts <- rep(1, nrow(data)); data$always1 <- data$always2 <- rep(FALSE, nrow(data)) # only allows subject with probabilities between 0<p<1 to be further selected
	if (method=='weighted' | (method=='offset' & !addStilde)) niter <- 1

 #  Model of interest
    M1 = model.apply(regformula,data,counts,always1)
    if (attr(M1$Terms,"response") != 1) stop("regformula must have a response variable")
    y = M1$y; X=M1$X; counts=M1$counts; always1=M1$always1

 #  Design Model 1
    M2 = model.apply(desnformula1,data,counts,always2)
    if (attr(M2$Terms,"response") != 1) stop("desnformula1 must have a response variable")
    R1 = M2$y; z1=M2$X; always2=M2$always1; R1aux = R1; z1aux = z1

 #  Design Model 1 Auxiliar
    if (!is.null(desnformula1)) {
      dataaux = data; dataaux$cc = NA
      M2 = model.apply(desnformula1,dataaux,counts)
      Z1=M2$X
      respcols1 = rep(0,ncol(Z1))
      for (c in 1:ncol(Z1))
        if (sum(is.na(Z1[,c])) == nrow(Z1)) respcols1[c] = 1
      badcols1 = rep(0,ncol(Z1))
      badcols1[respcols1==1] = as.numeric((colSums(na.omit(z1[y==1&R1==1,respcols1==1])) / colSums(na.omit(z1[y==1,respcols1==1])))>.99 |
                               abs(colSums(na.omit(z1[y==1&R1==1,respcols1==1]))) < 1)
    }

 #  Design Model 2
    if (!is.null(desnformula2)) {
      M3 = model.apply(desnformula2,data,counts)
      if (attr(M3$Terms,"response") != 1) stop("desnformula2 must have a response variable")
      R2 = M3$y; z2=as.matrix(M3$X)
    }

 #  Design Model 2 Auxiliar
    if (!is.null(desnformula2)) {
      dataaux = data; dataaux$cc = NA
      M3 = model.apply(desnformula2,dataaux,counts)
      if (attr(M3$Terms,"response") != 1) stop("desnformula2 must have a response variable")
      Z2=M3$X
      respcols = rep(0,ncol(Z2))
      for (c in 1:ncol(Z2))
        if (sum(is.na(Z2[,c])) == nrow(Z2)) respcols[c] = 1
      badcols = rep(0,ncol(Z2))
      badcols[respcols==1] = as.numeric((colSums(na.omit(z2[y==1&R1==1&R2==1,respcols==1])) / colSums(na.omit(z2[y==1&R1==1,respcols==1])))>.99 |
                             abs(colSums(na.omit(z2[y==1&R1==1&R2==1,respcols==1]))) < 1)
    }

 #  Design Model 3
    if (!is.null(desnformula3)) {
      M4 = model.apply(desnformula3,data,counts)
      if (attr(M4$Terms,"response") != 1) stop("desnformula3 must have a response variable")
      R3 = M4$y; z3=M4$X
    }

    use1 = !always1; fit1 = fit2 = fit3 = 0; use1 = rep(TRUE,length(R1))
    fit1 = glm(R1 ~ z1-1, family=binomial, weights=counts, subset=use1, data=data)
    alpha1 = as.vector(coefficients(fit1)); names(alpha1) = colnames(z1); pi1 = plogis(z1 %*% alpha1)

    if (!is.null(desnformula2)) {
      use2 = (R1==1 & !is.na(R2))
      fit2 = glm(R2 ~ z2-1, family=binomial, weights=counts, subset=use2, data=data)
      alpha2 = as.vector(coefficients(fit2)); names(alpha2) = colnames(z2); pi2 = plogis(z2 %*% alpha2)
    }

    if (!is.null(desnformula3)) {
      use3 = (R1==1 & R2==1)
      fit3 = glm(R3 ~ z3-1, family=binomial, weights=counts, subset=use3, data=data)
      alpha3 = as.vector(coefficients(fit3)); pi3 = plogis(z3 %*% alpha3)
    }

    if (method=="offset") {
      yvarname= names(M1[1])
      yvarname= "cc"
      temdata=data
      temdata[,yvarname] = rep(1,nrow(data)) # Make y-val always =1
      z1.1 = model.apply(desnformula1,temdata,counts)$X
      if (!is.null(desnformula2)) z2.1 = model.apply(desnformula2,temdata,counts)$X
      if (!is.null(desnformula3)) z3.1 = model.apply(desnformula3,temdata,counts)$X
      temdata[,yvarname] = rep(0,nrow(data)) # Make y-val always =0
      z1.0 = model.apply(desnformula1,temdata,counts)$X
      if (!is.null(desnformula2)) z2.0 = model.apply(desnformula2,temdata,counts)$X
      if (!is.null(desnformula3)) z3.0 = model.apply(desnformula3,temdata,counts)$X

      lpi1.1 = plogis(z1.1 %*% alpha1, log.p=TRUE)
      lpi1.0 = plogis(z1.0 %*% alpha1, log.p=TRUE)
      oo = lpi1.1 - lpi1.0
      if (!is.null(desnformula2)) {
        lpi2.1 = plogis(z2.1 %*% alpha2, log.p=TRUE)
        lpi2.0 = plogis(z2.0 %*% alpha2, log.p=TRUE)
        oo = lpi1.1 + lpi2.1 - lpi1.0 - lpi2.0
      }
      if (!is.null(desnformula3)) {
        lpi3.1 = plogis(z3.1 %*% alpha3, log.p=TRUE)
        lpi3.0 = plogis(z3.0 %*% alpha3, log.p=TRUE)
        oo = lpi1.1 + lpi2.1 + lpi3.1 - lpi1.0 - lpi2.0 - lpi3.0
      }
      weights = counts
    } else if (method=="weighted"){
      oo = rep(0,nrow(data))
      pi1pi2pi3 = as.vector(pi1)
      if (!is.null(desnformula2)) pi1pi2pi3 = as.vector(pi1*pi2)
      if (!is.null(desnformula3)) pi1pi2pi3 = as.vector(pi1*pi2*pi3)
      weights = R1/pi1pi2pi3 + (1-R1)/(1-pi1pi2pi3)
      options(warn = -1) # supress warnings (complains about non-integer weights)
    } else stop("Method not available")

    use0 = (R1==1)# those who have full data
    if (!is.null(desnformula2)) use0 = (R1==1 & R2==1)
    if (!is.null(desnformula3)) use0 = (R1==1 & R2==1 & R3==1)
    fit = glm(y ~ X-1, family=binomial, weights=weights, offset=oo, subset=use0)
    coefs = coefficients(fit); names(coefs)=colnames(X)
    options(ow) # reset

    if (!is.null(desnformula1) & sum(always1)>0) {
      z1.old = z1; z1.0.old = z1.0; z1.1.old = z1.1; alpha1.old = alpha1
      eq1 = (badcols1==FALSE)
      z1.new = as.matrix(z1.old[,c(eq1)]); alpha1.new = alpha1.old[c(eq1)]; alpha1 = alpha1.new
      z1.1.new = z1.1.old[,c(eq1)]; z1.0.new = z1.0.old[,c(eq1)]
      z1 = z1.new; z1.0 = z1.0.new; z1.1 = z1.1.new
    }

    error = 0; conv = FALSE
    n0=ncol(X); n1=ncol(z1)
    phi = c(coefs,alpha1)

    if (!is.null(desnformula2) & sum(always2)>0) {
      z2.old = z2; z2.0.old = z2.0; z2.1.old = z2.1; alpha2.old = alpha2
      eq = (badcols==FALSE)
      z2.new = as.matrix(z2.old[,c(eq)]); alpha2.new = alpha2.old[c(eq)]; alpha2 = alpha2.new
      z2.1.new = z2.1.old[,c(eq)]; z2.0.new = z2.0.old[,c(eq)]
      z2 = z2.new; z2.0 = z2.0.new; z2.1 = z2.1.new
    }

    if (!is.null(desnformula2)) {
      n2=ncol(z2)
      phi = c(coefs,alpha1,alpha2)
    }
    if (!is.null(desnformula3)) {
      n3=ncol(z3)
      phi = c(coefs,alpha1,alpha2,alpha3)
    }

    Step.zero = c(rep(0,length(coefs)),badcols1)
    if (!is.null(desnformula2))
      Step.zero = c(rep(0,length(coefs)),badcols1,badcols)

    for (iter in 1:niter){ # niter should = 1 except when addStilde=TRUE

      if (addStilde) {
        if (sum(always1)>0) eq1 = (badcols1==FALSE)
        coefs = phi[1:n0]
        alpha1 = phi[n0+(1:n1)]
        pi1 = plogis(z1 %*% alpha1)
        if (sum(always1)>0) {
          alpha1.new = alpha1
          alpha1.old[c(eq1)] = alpha1.new; alpha1 = alpha1.old
          z1 = z1.old; z1.0 = z1.0.old; z1.1 = z1.1.old
          z1 = as.matrix(z1); z1.1 = as.matrix(z1.1); z1.0 = as.matrix(z1.0)
          pi1 = plogis(z1 %*% alpha1)
        }

        if (!is.null(desnformula2)) {
          alpha2 = phi[n0+n1+(1:n2)]
          if (sum(always2)>0) {
            alpha2.new = alpha2
            alpha2.old[c(eq)] = alpha2.new; alpha2 = alpha2.old
            z2 = z2.old; z2.0 = z2.0.old; z2.1 = z2.1.old
          }
          pi2 = plogis(z2 %*% alpha2)
        }
        if (!is.null(desnformula3)) {
          alpha3 = phi[n0+n1+n2+(1:n3)]
          pi3 = plogis(z3 %*% alpha3)
        }
        if (method=="offset"){
          lpi1.1 = plogis(z1.1 %*% alpha1, log.p=TRUE)
          lpi1.0 = plogis(z1.0 %*% alpha1, log.p=TRUE)
          oo = lpi1.1 - lpi1.0
          if (!is.null(desnformula2)) {
            lpi2.1 = plogis(z2.1 %*% alpha2, log.p=TRUE)
            lpi2.0 = plogis(z2.0 %*% alpha2, log.p=TRUE)
            oo = lpi1.1 + lpi2.1 - lpi1.0 - lpi2.0
          }
          if (!is.null(desnformula3)) {
            lpi3.1 = plogis(z3.1 %*% alpha3, log.p=TRUE)
            lpi3.0 = plogis(z3.0 %*% alpha3, log.p=TRUE)
            oo = lpi1.1 + lpi2.1 + lpi3.1 - lpi1.0 - lpi2.0 - lpi3.0
          }
        }
        if (sum(always1)>0) {
          z1 = z1.new; z1.0 = z1.0.new; z1.1 = z1.1.new; alpha1 = alpha1.new
          z1 = as.matrix(z1); z1.1 = as.matrix(z1.1); z1.0 = as.matrix(z1.0)
        }
        if (!is.null(desnformula2)&sum(always2)>0) {
          z2 = z2.new; z2.0 = z2.0.new; z2.1 = z2.1.new; alpha2 = alpha2.new
        }
      }
      S1  = z1*(R1-as.vector(pi1)) #colSums(S1[use1,])
      pi1vec = as.vector(pi1);
      if (!is.null(desnformula2)) {
        S2  = z2*(R2-as.vector(pi2)) #colSums(S2[use2,])
        pi2vec = as.vector(pi2)
      }
      if (!is.null(desnformula3)) {
        S3  = z3*(R3-as.vector(pi3)) #colSums(S3[use3,])
        pi3vec = as.vector(pi3)
      }

      if (method=="offset"){
        pstar = as.vector(plogis(oo+X%*%coefs))
        S0    = X*(y-pstar)
        I00   = t(X[use0,,drop=FALSE]) %*% diag((counts[use0]*pstar[use0]*(1-pstar[use0]))) %*% X[use0,,drop=FALSE]
        pi1.1     = as.vector(exp(lpi1.1)); pi1.0 = as.vector(exp(lpi1.0))
        dodalpha1 = as.matrix((z1.1*(1-pi1.1) - z1.0*(1-pi1.0)))
        I01   = t(X[use0,,drop=FALSE]) %*% diag((counts[use0]*pstar[use0]*(1-pstar[use0]))) %*% dodalpha1[use0,,drop=FALSE]
        z1aux = z1*sqrt(counts*pi1vec*(1-pi1vec))
        I11   = t(z1aux) %*% z1aux
        I10 = mzero(n1,n0)
        II  = rbind(cbind(I00,I01),
                    cbind(I10,I11))
        S = t(cbind(t(colSums((counts*S0)[use0,,drop=FALSE])),
            t(colSums((counts*S1)[use1,,drop=FALSE])) ))

        if (!is.null(desnformula2)) {
          pi2.1     = as.vector(exp(lpi2.1)); pi2.0 = as.vector(exp(lpi2.0))
          dodalpha2 = (z2.1*(1-pi2.1) - z2.0*(1-pi2.0))
          I02   = t(X[use0,,drop=FALSE]) %*% diag((counts[use0]*pstar[use0]*(1-pstar[use0]))) %*% dodalpha2[use0,,drop=FALSE]
          z2aux = (z2*sqrt(counts*pi2vec*(1-pi2vec)))[use2,]
          I22   = t(z2[use2,,drop=FALSE]) %*% diag((counts*pi2*(1-pi2))[use2]) %*% z2[use2,,drop=FALSE]
          I20 = mzero(n2,n0); I12 = mzero(n1,n2); I21 = t(I12)
          II  = rbind(cbind(I00,I01,I02),
                      cbind(I10,I11,I12),
                      cbind(I20,I21,I22))
          S = t(cbind(t(colSums((counts*S0)[use0,,drop=FALSE])),
              t(colSums((counts*S1)[use1,,drop=FALSE])),
              t(colSums((counts*S2)[use2,,drop=FALSE])) ))
        }
        if (!is.null(desnformula3)) {
          pi3.1     = as.vector(exp(lpi3.1)); pi3.0 = as.vector(exp(lpi3.0))
          dodalpha3 = (z3.1*(1-pi3.1) - z3.0*(1-pi3.0))
          I03   = t(X[use0,,drop=FALSE]) %*% diag((counts[use0]*pstar[use0]*(1-pstar[use0]))) %*% dodalpha3[use0,,drop=FALSE]
          z3aux = (z3*sqrt(counts*pi3vec*(1-pi3vec)))[use3,]
          I33   = t(z3aux) %*% z3aux
          I30 = mzero(n3,n0); I13 = mzero(n1,n3);  I23 = mzero(n2,n3); I31 = t(I13); I32 = t(I23)
          II  = rbind(cbind(I00,I01,I02,I03),
                      cbind(I10,I11,I12,I13),
                      cbind(I20,I21,I22,I23),
                      cbind(I30,I31,I32,I33))
          S = t(cbind(t(colSums((counts*S0)[use0,,drop=FALSE])),
              t(colSums((counts*S1)[use1,,drop=FALSE])),
              t(colSums((counts*S2)[use2,,drop=FALSE])),
              t(colSums((counts*S3)[use3,,drop=FALSE])) ))
        }

        if (addStilde){
          Stilde1 = dodalpha1*(y-pstar) # Subsetting done later when required
          Itilde10 = t(dodalpha1[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% X[use0,,drop=FALSE]
          Itilde11 = t(dodalpha1[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% dodalpha1[use0,,drop=FALSE] +
                     t(z1.1[use0,,drop=FALSE]) %*% diag((counts*pi1.1*(1-pi1.1)*(y-pstar))[use0]) %*% z1.1[use0,,drop=FALSE] -
                     t(z1.0[use0,,drop=FALSE]) %*% diag((counts*pi1.0*(1-pi1.0)*(y-pstar))[use0]) %*% z1.0[use0,,drop=FALSE]
          I10 = I10 - Itilde10; I11 = I11 - Itilde11;
          II  = rbind(cbind(I00,I01),
                      cbind(I10,I11))
          S = t(cbind(t(colSums((counts*S0)[use0,,drop=FALSE])),
              t(colSums((counts*S1)[use1,,drop=FALSE]) - colSums((counts*Stilde1)[use0,,drop=FALSE])) ))

          if (!is.null(desnformula2)) {
            Stilde2 = dodalpha2*(y-pstar) # Subsetting done later when required
            Itilde20 = t(dodalpha2[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% X[use0,,drop=FALSE]
            Itilde12 = t(dodalpha1[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% dodalpha2[use0,,drop=FALSE]
            Itilde22 = t(dodalpha2[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% dodalpha2[use0,,drop=FALSE] +
                       t(z2.1[use0,,drop=FALSE]) %*% diag((counts*pi2.1*(1-pi2.1)*(y-pstar))[use0]) %*% z2.1[use0,,drop=FALSE] -
                       t(z2.0[use0,,drop=FALSE]) %*% diag((counts*pi2.0*(1-pi2.0)*(y-pstar))[use0]) %*% z2.0[use0,,drop=FALSE]
            Itilde21 = t(Itilde12)
            I12 = I12 - Itilde12; I20 = I20 - Itilde20; I21 = I21 - Itilde21; I22 = I22 - Itilde22
            II  = rbind(cbind(I00,I01,I02),
                        cbind(I10,I11,I12),
                        cbind(I20,I21,I22))
            S = t(cbind(t(colSums((counts*S0)[use0,,drop=FALSE])),
                t(colSums((counts*S1)[use1,,drop=FALSE]) - colSums((counts*Stilde1)[use0,,drop=FALSE])),
                t(colSums((counts*S2)[use2,,drop=FALSE]) - colSums((counts*Stilde2)[use0,,drop=FALSE])) ))
          }

          if (!is.null(desnformula3)) {
            Stilde3 = dodalpha3*(y-pstar) # Subsetting done later when required
            Itilde30 = t(dodalpha3[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% X[use0,,drop=FALSE]
            Itilde13 = t(dodalpha1[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% dodalpha3[use0,,drop=FALSE]
            Itilde23 = t(dodalpha2[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% dodalpha3[use0,,drop=FALSE]
            Itilde33 = t(dodalpha3[use0,,drop=FALSE]) %*% diag((counts*pstar*(1-pstar))[use0]) %*% dodalpha3[use0,,drop=FALSE] +
                       t(z3.1[use0,,drop=FALSE]) %*% diag((counts*pi3.1*(1-pi3.1)*(y-pstar))[use0]) %*% z3.1[use0,,drop=FALSE] -
                       t(z3.0[use0,,drop=FALSE]) %*% diag((counts*pi3.0*(1-pi3.0)*(y-pstar))[use0]) %*% z3.0[use0,,drop=FALSE]
            Itilde31 = t(Itilde13); Itilde32 = t(Itilde23)
            I30 = I30 - Itilde30; I31 = I31 - Itilde31; I32 = I32 - Itilde32; I33 = I33 - Itilde33; I13 = I13 - Itilde13; I23 = I23 - Itilde23
            II  = rbind(cbind(I00,I01,I02,I03),
                        cbind(I10,I11,I12,I13),
                        cbind(I20,I21,I22,I23),
                        cbind(I30,I31,I32,I33))
            S = t(cbind(t(colSums((counts*S0)[use0,,drop=FALSE])),
                t(colSums((counts*S1)[use1,,drop=FALSE]) - colSums((counts*Stilde1)[use0,,drop=FALSE])),
                t(colSums((counts*S2)[use2,,drop=FALSE]) - colSums((counts*Stilde2)[use0,,drop=FALSE])),
                t(colSums((counts*S3)[use3,,drop=FALSE]) - colSums((counts*Stilde3)[use0,,drop=FALSE])) ))
          }
        } #End addStilde
      } else if (method=="weighted"){
        p   = as.vector(plogis(X%*%coefs))
        S0  = X*((y-p)/pi1pi2pi3) # colSums(S0[use0,,drop=FALSE])
        I10 = mzero(n1,n0)
        I00 = t(X[use0,,drop=FALSE]) %*% diag((counts*p*(1-p)/pi1pi2pi3)[use0]) %*% X[use0,,drop=FALSE]
        I01 = t(S0[use0,,drop=FALSE]) %*% diag((counts*(1-pi1vec))[use0]) %*% z1[use0,,drop=FALSE]
        z1aux = z1*sqrt(counts*pi1vec*(1-pi1vec))
        I11 = t(z1aux) %*% z1aux
        if (!is.null(desnformula2)) {
          I20 = mzero(n2,n0); I12 = mzero(n1,n2); I21 = t(I12)
          I02 = t(S0[use0,,drop=FALSE]) %*% diag((counts*(1-pi2vec))[use0]) %*% z2[use0,,drop=FALSE]
          z2aux = z2*sqrt(counts*pi2vec*(1-pi2vec))
          I22 = t(z2aux[use2,]) %*% z2aux[use2,]
        }
        if (!is.null(desnformula3)) {
          I30 = mzero(n3,n0); I13 = mzero(n1,n3);  I23 = mzero(n2,n3); I31 = t(I13); I32 = t(I23)
          I03 = t(S0[use0,,drop=FALSE]) %*% diag((counts*(1-pi3vec))[use0]) %*% z3[use0,,drop=FALSE]
          z3aux = z3*sqrt(counts*pi3vec*(1-pi3vec))
          I33 = t(z3aux) %*% z3aux
        }
      } else stop("Method not available")

      Step = 0
      if (niter >1) Step = solve(II,S)
      bigstep = max(abs(Step))
      maxstep = maxstepprop*max(abs(phi))
      if (bigstep > maxstep) {
          Step = Step * maxstep/bigstep
          if (print.iters) print(paste("Step shortened by",maxstep/bigstep))
      }

     Step[Step.zero==1] = 0
      phi2 = phi + Step
      if (print.iters & niter>1) {
        print(paste("Iter", iter))
        print(paste("phi= ",phi))
        print(paste("Step= ",Step))
        print(paste("S=", S))
      }

      if (niter>1) if (max(abs(phi-phi2)[1:ncol(X)]) < tol*max(abs(phi[1:ncol(X)]))){#& max(abs(S)[1:ncol(X)]) < tol) {
          conv = TRUE
          break
      }
      phi = phi2
    } # END OF ITERATION LOOP
    if (niter>1 & !conv) return(invisible(list(msg="Failed to Converge", phi=phi,S=S,II=II,error=1,pi1=pi1)))

    if (addStilde) {
        dimS1 = dim(S1)
        lines0 = (1:nrow(S1))[use0]
        S1tilde = matrix(rep(0, dim(S1)[1]*dim(S1)[2]), nrow = dim(S1)[1])
        S1tilde[lines0,] = Stilde1[use0,,drop=FALSE]
        names = colnames(S1)
        S1 = S1 - S1tilde #colSums(S1[use1,])
        colnames(S1) = names

        if (!is.null(desnformula2)) {
          lines2 = (1:nrow(S1))[use2]
          S2star = matrix(rep(0, dim(S1)[1]*dim(S2)[2]), nrow = dim(S1)[1])
          S2star[lines2,] = S2[use2,,drop=FALSE]
          linesS2 = (1:nrow(S1))[use0]
          S2tilde = matrix(rep(0, dim(S1)[1]*dim(S2star)[2]), nrow = dim(S1)[1])
          S2tilde[linesS2,] = Stilde2[use0,,drop=FALSE]
          S2 = S2star - S2tilde #colSums(S2[use2,])
          colnames(S2) = colnames(Stilde2)
        }
    }

    C00 = empcov(S0[use0,,drop=FALSE],nreps=counts[use0])$SS
    C01 = empcov(x=S0[use0,,drop=FALSE],y=S1[use0,,drop=FALSE],nreps=counts[use0])$SS
    C11 = empcov(S1[use1,,drop=FALSE],nreps=counts[use1])$SS
    CC = rbind(cbind(C00,C01),cbind(t(C01),C11))
    if (!is.null(desnformula2)) {
      C02 = empcov(x=S0[use0,,drop=FALSE],y=S2[use0,,drop=FALSE],nreps=counts[use0])$SS
      C12 = empcov(x=S1[use2,,drop=FALSE],y=S2[use2,,drop=FALSE],nreps=counts[use2])$SS
      C22 = empcov(S2[use2,,drop=FALSE],nreps=counts[use2])$SS
      CC = rbind(cbind(C00,C01,C02),cbind(t(C01),C11,C12),cbind(t(C02),t(C12),C22))
    }
    if (method=="weighted") {
      Mid = C00 - I01%*%solve(I11,t(I01))
      if (!is.null(desnformula2)) Mid = C00 - I01%*%solve(I11,t(I01)) - I02%*%solve(I22,t(I02))
      Var = solve(I00,t(solve(I00,Mid)))
    } else Var = solve(II,t(solve(II,CC)))
    ses = sqrt((diag(Var))[1:ncol(X)])

    ll <- coefs + qnorm(.025)*ses; ul <- coefs + qnorm(.975)*ses
    results = round(matrix(c(coefs, exp(coefs), ses, ll, ul, exp(ll), exp(ul)),ncol=7),4)
    colnames(results) = c("Estimates", "OR", "ses", "ll", "ul", "OR.ll", "OR.ul"); rownames(results) = colnames(X)

    invisible(list(coefficients=coefs,cov=Var,fit=fit,desfit=fit1,fit2=fit2,fit3=fit3,error=error,results=results))
}

